import { API as API$1 } from '../../../scw/api.js';
import { urlParams, validatePathParam } from '../../../helpers/marshalling.js';
import { enrichForPagination } from '../../../scw/fetch/resource-paginator.js';
import { marshalProjectApiCreateProjectRequest, unmarshalProject, unmarshalListProjectsResponse, marshalProjectApiUpdateProjectRequest } from './marshalling.gen.js';

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const jsonContentHeaders = {
  'Content-Type': 'application/json; charset=utf-8'
};

/**
 * Account API.
 *
 * This API allows you to manage projects.
 */
class ProjectAPI extends API$1 {
  /**
   * Create a new Project for an Organization. Generate a new Project for an
   * Organization, specifying its configuration including name and description.
   *
   * @param request - The request {@link ProjectApiCreateProjectRequest}
   * @returns A Promise of Project
   */
  createProject = request => this.client.fetch({
    body: JSON.stringify(marshalProjectApiCreateProjectRequest(request, this.client.settings)),
    headers: jsonContentHeaders,
    method: 'POST',
    path: `/account/v3/projects`
  }, unmarshalProject);
  pageOfListProjects = (() => {
    var _this = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return _this.client.fetch({
        method: 'GET',
        path: `/account/v3/projects`,
        urlParams: urlParams(['name', request.name], ['order_by', request.orderBy], ['organization_id', request.organizationId ?? _this.client.settings.defaultOrganizationId], ['page', request.page], ['page_size', request.pageSize ?? _this.client.settings.defaultPageSize], ['project_ids', request.projectIds])
      }, unmarshalListProjectsResponse);
    };
  })();

  /**
   * List all Projects of an Organization. List all Projects of an Organization.
   * The response will include the total number of Projects as well as their
   * associated Organizations, names, and IDs. Other information includes the
   * creation and update date of the Project.
   *
   * @param request - The request {@link ProjectApiListProjectsRequest}
   * @returns A Promise of ListProjectsResponse
   */
  listProjects = (() => {
    var _this2 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return enrichForPagination('projects', _this2.pageOfListProjects, request);
    };
  })();

  /**
   * Get an existing Project. Retrieve information about an existing Project,
   * specified by its Project ID. Its full details, including ID, name and
   * description, are returned in the response object.
   *
   * @param request - The request {@link ProjectApiGetProjectRequest}
   * @returns A Promise of Project
   */
  getProject = (() => {
    var _this3 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return _this3.client.fetch({
        method: 'GET',
        path: `/account/v3/projects/${validatePathParam('projectId', request.projectId ?? _this3.client.settings.defaultProjectId)}`
      }, unmarshalProject);
    };
  })();

  /**
   * Delete an existing Project. Delete an existing Project, specified by its
   * Project ID. The Project needs to be empty (meaning there are no resources
   * left in it) to be deleted effectively. Note that deleting a Project is
   * permanent, and cannot be undone.
   *
   * @param request - The request {@link ProjectApiDeleteProjectRequest}
   */
  deleteProject = (() => {
    var _this4 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return _this4.client.fetch({
        method: 'DELETE',
        path: `/account/v3/projects/${validatePathParam('projectId', request.projectId ?? _this4.client.settings.defaultProjectId)}`
      });
    };
  })();

  /**
   * Update Project. Update the parameters of an existing Project, specified by
   * its Project ID. These parameters include the name and description.
   *
   * @param request - The request {@link ProjectApiUpdateProjectRequest}
   * @returns A Promise of Project
   */
  updateProject = (() => {
    var _this5 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return _this5.client.fetch({
        body: JSON.stringify(marshalProjectApiUpdateProjectRequest(request, _this5.client.settings)),
        headers: jsonContentHeaders,
        method: 'PATCH',
        path: `/account/v3/projects/${validatePathParam('projectId', request.projectId ?? _this5.client.settings.defaultProjectId)}`
      }, unmarshalProject);
    };
  })();
}

export { ProjectAPI };
