import { API as API$1 } from '../../../scw/api.js';
import { urlParams, validatePathParam } from '../../../helpers/marshalling.js';
import { enrichForPagination } from '../../../scw/fetch/resource-paginator.js';
import { unmarshalGetConsumptionResponse, unmarshalListInvoicesResponse } from './marshalling.gen.js';

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
/**
 * Billing API.
 *
 * This API allows you to query your consumption.
 */
class API extends API$1 {
  /**
   * Get current month's consumption. The consumption reflects the amount of
   * money you have spent for the products you have used. The consumption value
   * is monetary and is not computed in real time.
   *
   * @param request - The request {@link GetConsumptionRequest}
   * @returns A Promise of GetConsumptionResponse
   */
  getConsumption = (() => {
    var _this = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return _this.client.fetch({
        method: 'GET',
        path: `/billing/v2alpha1/consumption`,
        urlParams: urlParams(['organization_id', request.organizationId ?? _this.client.settings.defaultOrganizationId])
      }, unmarshalGetConsumptionResponse);
    };
  })();
  pageOfListInvoices = (() => {
    var _this2 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return _this2.client.fetch({
        method: 'GET',
        path: `/billing/v2alpha1/invoices`,
        urlParams: urlParams(['invoice_type', request.invoiceType], ['order_by', request.orderBy], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize ?? _this2.client.settings.defaultPageSize], ['started_after', request.startedAfter], ['started_before', request.startedBefore])
      }, unmarshalListInvoicesResponse);
    };
  })();

  /**
   * List invoices. List all your invoices, filtering by `start_date` and
   * `invoice_type`. Each invoice has its own ID.
   *
   * @param request - The request {@link ListInvoicesRequest}
   * @returns A Promise of ListInvoicesResponse
   */
  listInvoices = (() => {
    var _this3 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return enrichForPagination('invoices', _this3.pageOfListInvoices, request);
    };
  })();

  /**
   * Download an invoice. Download a specific invoice, specified by its ID.
   *
   * @param request - The request {@link DownloadInvoiceRequest}
   * @returns A Promise of Blob
   */
  downloadInvoice = request => this.client.fetch({
    method: 'GET',
    path: `/billing/v2alpha1/invoices/${validatePathParam('invoiceId', request.invoiceId)}/download`,
    urlParams: urlParams(['dl', 1], ['file_type', request.fileType]),
    responseType: 'blob'
  });
}

export { API };
