import { waitForResource } from '../../../internal/async/interval-retrier.js';
import { API as API$1 } from '../../../scw/api.js';
import { validatePathParam, urlParams } from '../../../helpers/marshalling.js';
import { enrichForPagination } from '../../../scw/fetch/resource-paginator.js';
import { VOLUME_TRANSIENT_STATUSES, SNAPSHOT_TRANSIENT_STATUSES } from './content.gen.js';
import { unmarshalListVolumeTypesResponse, unmarshalListVolumesResponse, marshalCreateVolumeRequest, unmarshalVolume, marshalUpdateVolumeRequest, unmarshalListSnapshotsResponse, unmarshalSnapshot, marshalCreateSnapshotRequest, marshalUpdateSnapshotRequest } from './marshalling.gen.js';

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const jsonContentHeaders = {
  'Content-Type': 'application/json; charset=utf-8'
};

/**
 * Scaleway Block Storage (SBS) API.
 *
 * This API allows you to use and manage your Block Storage volumes. Scaleway
 * Block Storage (SBS) API.
 */
class API extends API$1 {
  /** Lists the available zones of the API. */
  static LOCALITIES = ['fr-par-1', 'pl-waw-3'];
  pageOfListVolumeTypes = (() => {
    var _this = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return _this.client.fetch({
        method: 'GET',
        path: `/block/v1alpha1/zones/${validatePathParam('zone', request.zone ?? _this.client.settings.defaultZone)}/volume-types`,
        urlParams: urlParams(['page', request.page], ['page_size', request.pageSize ?? _this.client.settings.defaultPageSize])
      }, unmarshalListVolumeTypesResponse);
    };
  })();

  /**
   * List volume types. List all available volume types in a specified zone. The
   * volume types listed are ordered by name in ascending order.
   *
   * @param request - The request {@link ListVolumeTypesRequest}
   * @returns A Promise of ListVolumeTypesResponse
   */
  listVolumeTypes = (() => {
    var _this2 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return enrichForPagination('volumeTypes', _this2.pageOfListVolumeTypes, request);
    };
  })();
  pageOfListVolumes = (() => {
    var _this3 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return _this3.client.fetch({
        method: 'GET',
        path: `/block/v1alpha1/zones/${validatePathParam('zone', request.zone ?? _this3.client.settings.defaultZone)}/volumes`,
        urlParams: urlParams(['name', request.name], ['order_by', request.orderBy ?? 'created_at_asc'], ['page', request.page], ['page_size', request.pageSize ?? _this3.client.settings.defaultPageSize], ['product_resource_id', request.productResourceId], ['project_id', request.projectId])
      }, unmarshalListVolumesResponse);
    };
  })();

  /**
   * List volumes. List all existing volumes in a specified zone. By default,
   * the volume listed are ordered by creation date in ascending order. This can
   * be modified via the `order_by` field.
   *
   * @param request - The request {@link ListVolumesRequest}
   * @returns A Promise of ListVolumesResponse
   */
  listVolumes = (() => {
    var _this4 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return enrichForPagination('volumes', _this4.pageOfListVolumes, request);
    };
  })();

  /**
   * Create a new empty volume by specifying the `size`. To create a volume from
   * an existing snapshot, specify the `snapshot_id` in the request payload
   * instead, size is optional and can be specified if you need to extend the
   * original size. In that case the created volume will have the same volume
   * class (and underlying IOPS limitations) as the originating snapshot. You
   * can specify the desired performance of the volume by setting `requirements`
   * accordingly.
   *
   * @param request - The request {@link CreateVolumeRequest}
   * @returns A Promise of Volume
   */
  createVolume = request => this.client.fetch({
    body: JSON.stringify(marshalCreateVolumeRequest(request, this.client.settings)),
    headers: jsonContentHeaders,
    method: 'POST',
    path: `/block/v1alpha1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/volumes`
  }, unmarshalVolume);

  /**
   * Get a volume. Retrieve technical information about a specific volume.
   * Details such as size, type, and status are returned in the response.
   *
   * @param request - The request {@link GetVolumeRequest}
   * @returns A Promise of Volume
   */
  getVolume = request => this.client.fetch({
    method: 'GET',
    path: `/block/v1alpha1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/volumes/${validatePathParam('volumeId', request.volumeId)}`
  }, unmarshalVolume);

  /**
   * Waits for {@link Volume} to be in a final state.
   *
   * @param request - The request {@link GetVolumeRequest}
   * @param options - The waiting options
   * @returns A Promise of Volume
   */
  waitForVolume = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!VOLUME_TRANSIENT_STATUSES.includes(res.status))), this.getVolume, request, options);

  /**
   * Delete a detached volume. You must specify the `volume_id` of the volume
   * you want to delete. The volume must not be in the `in_use` status.
   *
   * @param request - The request {@link DeleteVolumeRequest}
   */
  deleteVolume = request => this.client.fetch({
    method: 'DELETE',
    path: `/block/v1alpha1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/volumes/${validatePathParam('volumeId', request.volumeId)}`
  });

  /**
   * Update a volume. Update technical details about a volume, such as its name,
   * tags, or its new size and `volume_type` (within the same Block Storage
   * class). You can only resize a volume to a larger size. It is not possible
   * for now to change your Block Storage Class.
   *
   * @param request - The request {@link UpdateVolumeRequest}
   * @returns A Promise of Volume
   */
  updateVolume = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateVolumeRequest(request, this.client.settings)),
    headers: jsonContentHeaders,
    method: 'PATCH',
    path: `/block/v1alpha1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/volumes/${validatePathParam('volumeId', request.volumeId)}`
  }, unmarshalVolume);
  pageOfListSnapshots = (() => {
    var _this5 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return _this5.client.fetch({
        method: 'GET',
        path: `/block/v1alpha1/zones/${validatePathParam('zone', request.zone ?? _this5.client.settings.defaultZone)}/snapshots`,
        urlParams: urlParams(['name', request.name], ['order_by', request.orderBy ?? 'created_at_asc'], ['page', request.page], ['page_size', request.pageSize ?? _this5.client.settings.defaultPageSize], ['project_id', request.projectId], ['volume_id', request.volumeId])
      }, unmarshalListSnapshotsResponse);
    };
  })();

  /**
   * List all snapshots. List all available snapshots in a specified zone. By
   * default, the snapshots listed are ordered by creation date in ascending
   * order. This can be modified via the `order_by` field.
   *
   * @param request - The request {@link ListSnapshotsRequest}
   * @returns A Promise of ListSnapshotsResponse
   */
  listSnapshots = (() => {
    var _this6 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return enrichForPagination('snapshots', _this6.pageOfListSnapshots, request);
    };
  })();

  /**
   * Get a snapshot. Retrieve technical information about a specific snapshot.
   * Details such as size, volume type, and status are returned in the
   * response.
   *
   * @param request - The request {@link GetSnapshotRequest}
   * @returns A Promise of Snapshot
   */
  getSnapshot = request => this.client.fetch({
    method: 'GET',
    path: `/block/v1alpha1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/snapshots/${validatePathParam('snapshotId', request.snapshotId)}`
  }, unmarshalSnapshot);

  /**
   * Waits for {@link Snapshot} to be in a final state.
   *
   * @param request - The request {@link GetSnapshotRequest}
   * @param options - The waiting options
   * @returns A Promise of Snapshot
   */
  waitForSnapshot = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!SNAPSHOT_TRANSIENT_STATUSES.includes(res.status))), this.getSnapshot, request, options);

  /**
   * Create a snapshot of a volume. To create a snapshot, the volume must be in
   * the `in_use` or the `available` status. If your volume is in a transient
   * state, you need to wait until the end of the current operation.
   *
   * @param request - The request {@link CreateSnapshotRequest}
   * @returns A Promise of Snapshot
   */
  createSnapshot = request => this.client.fetch({
    body: JSON.stringify(marshalCreateSnapshotRequest(request, this.client.settings)),
    headers: jsonContentHeaders,
    method: 'POST',
    path: `/block/v1alpha1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/snapshots`
  }, unmarshalSnapshot);

  /**
   * Delete a snapshot. You must specify the `snapshot_id` of the snapshot you
   * want to delete. The snapshot must not be in use.
   *
   * @param request - The request {@link DeleteSnapshotRequest}
   */
  deleteSnapshot = request => this.client.fetch({
    method: 'DELETE',
    path: `/block/v1alpha1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/snapshots/${validatePathParam('snapshotId', request.snapshotId)}`
  });

  /**
   * Update a snapshot. Update name or tags of the snapshot.
   *
   * @param request - The request {@link UpdateSnapshotRequest}
   * @returns A Promise of Snapshot
   */
  updateSnapshot = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateSnapshotRequest(request, this.client.settings)),
    headers: jsonContentHeaders,
    method: 'PATCH',
    path: `/block/v1alpha1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/snapshots/${validatePathParam('snapshotId', request.snapshotId)}`
  }, unmarshalSnapshot);
}

export { API };
