import { waitForResource } from '../../../internal/async/interval-retrier.js';
import { API as API$1 } from '../../../scw/api.js';
import { urlParams, validatePathParam } from '../../../helpers/marshalling.js';
import { enrichForPagination } from '../../../scw/fetch/resource-paginator.js';
import { COCKPIT_TRANSIENT_STATUSES } from './content.gen.js';
import { marshalActivateCockpitRequest, unmarshalCockpit, unmarshalCockpitMetrics, marshalDeactivateCockpitRequest, marshalResetCockpitGrafanaRequest, marshalCreateDatasourceRequest, unmarshalDatasource, marshalCreateTokenRequest, unmarshalToken, unmarshalListTokensResponse, marshalCreateContactPointRequest, unmarshalContactPoint, unmarshalListContactPointsResponse, marshalDeleteContactPointRequest, marshalEnableManagedAlertsRequest, marshalDisableManagedAlertsRequest, marshalTriggerTestAlertRequest, marshalCreateGrafanaUserRequest, unmarshalGrafanaUser, unmarshalListGrafanaUsersResponse, marshalDeleteGrafanaUserRequest, marshalResetGrafanaUserPasswordRequest, unmarshalListPlansResponse, marshalSelectPlanRequest, unmarshalSelectPlanResponse } from './marshalling.gen.js';

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const jsonContentHeaders = {
  'Content-Type': 'application/json; charset=utf-8'
};

/**
 * Cockpit API.
 *
 * Cockpit's API allows you to activate your Cockpit on your Projects.
 * Scaleway's Cockpit stores metrics and logs and provides a dedicated Grafana
 * for dashboarding to visualize them.
 */
class API extends API$1 {
  /**
   * Activate the Cockpit of the specified Project ID.
   *
   * @param request - The request {@link ActivateCockpitRequest}
   * @returns A Promise of Cockpit
   */
  activateCockpit = (() => {
    var _this = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return _this.client.fetch({
        body: JSON.stringify(marshalActivateCockpitRequest(request, _this.client.settings)),
        headers: jsonContentHeaders,
        method: 'POST',
        path: `/cockpit/v1beta1/activate`
      }, unmarshalCockpit);
    };
  })();

  /**
   * Retrieve the Cockpit of the specified Project ID.
   *
   * @param request - The request {@link GetCockpitRequest}
   * @returns A Promise of Cockpit
   */
  getCockpit = (() => {
    var _this2 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return _this2.client.fetch({
        method: 'GET',
        path: `/cockpit/v1beta1/cockpit`,
        urlParams: urlParams(['project_id', request.projectId ?? _this2.client.settings.defaultProjectId])
      }, unmarshalCockpit);
    };
  })();

  /**
   * Waits for {@link Cockpit} to be in a final state.
   *
   * @param request - The request {@link GetCockpitRequest}
   * @param options - The waiting options
   * @returns A Promise of Cockpit
   */
  waitForCockpit = (() => {
    var _this3 = this;
    return function (request, options) {
      if (request === void 0) {
        request = {};
      }
      return waitForResource(options?.stop ?? (res => Promise.resolve(!COCKPIT_TRANSIENT_STATUSES.includes(res.status))), _this3.getCockpit, request, options);
    };
  })();

  /**
   * Get metrics from your Cockpit with the specified Project ID.
   *
   * @param request - The request {@link GetCockpitMetricsRequest}
   * @returns A Promise of CockpitMetrics
   */
  getCockpitMetrics = (() => {
    var _this4 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return _this4.client.fetch({
        method: 'GET',
        path: `/cockpit/v1beta1/cockpit/metrics`,
        urlParams: urlParams(['end_date', request.endDate], ['metric_name', request.metricName], ['project_id', request.projectId ?? _this4.client.settings.defaultProjectId], ['start_date', request.startDate])
      }, unmarshalCockpitMetrics);
    };
  })();

  /**
   * Deactivate the Cockpit of the specified Project ID.
   *
   * @param request - The request {@link DeactivateCockpitRequest}
   * @returns A Promise of Cockpit
   */
  deactivateCockpit = (() => {
    var _this5 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return _this5.client.fetch({
        body: JSON.stringify(marshalDeactivateCockpitRequest(request, _this5.client.settings)),
        headers: jsonContentHeaders,
        method: 'POST',
        path: `/cockpit/v1beta1/deactivate`
      }, unmarshalCockpit);
    };
  })();

  /**
   * Reset your Cockpit's Grafana associated with the specified Project ID.
   *
   * @param request - The request {@link ResetCockpitGrafanaRequest}
   * @returns A Promise of Cockpit
   */
  resetCockpitGrafana = (() => {
    var _this6 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return _this6.client.fetch({
        body: JSON.stringify(marshalResetCockpitGrafanaRequest(request, _this6.client.settings)),
        headers: jsonContentHeaders,
        method: 'POST',
        path: `/cockpit/v1beta1/reset-grafana`
      }, unmarshalCockpit);
    };
  })();

  /**
   * Create a datasource for the specified Project ID and the given type.
   *
   * @param request - The request {@link CreateDatasourceRequest}
   * @returns A Promise of Datasource
   */
  createDatasource = request => this.client.fetch({
    body: JSON.stringify(marshalCreateDatasourceRequest(request, this.client.settings)),
    headers: jsonContentHeaders,
    method: 'POST',
    path: `/cockpit/v1beta1/datasources`
  }, unmarshalDatasource);

  /**
   * Create a token associated with the specified Project ID.
   *
   * @param request - The request {@link CreateTokenRequest}
   * @returns A Promise of Token
   */
  createToken = request => this.client.fetch({
    body: JSON.stringify(marshalCreateTokenRequest(request, this.client.settings)),
    headers: jsonContentHeaders,
    method: 'POST',
    path: `/cockpit/v1beta1/tokens`
  }, unmarshalToken);
  pageOfListTokens = (() => {
    var _this7 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return _this7.client.fetch({
        method: 'GET',
        path: `/cockpit/v1beta1/tokens`,
        urlParams: urlParams(['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? _this7.client.settings.defaultPageSize], ['project_id', request.projectId ?? _this7.client.settings.defaultProjectId])
      }, unmarshalListTokensResponse);
    };
  })();

  /**
   * Get a list of tokens associated with the specified Project ID.
   *
   * @param request - The request {@link ListTokensRequest}
   * @returns A Promise of ListTokensResponse
   */
  listTokens = (() => {
    var _this8 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return enrichForPagination('tokens', _this8.pageOfListTokens, request);
    };
  })();

  /**
   * Retrieve the token associated with the specified token ID.
   *
   * @param request - The request {@link GetTokenRequest}
   * @returns A Promise of Token
   */
  getToken = request => this.client.fetch({
    method: 'GET',
    path: `/cockpit/v1beta1/tokens/${validatePathParam('tokenId', request.tokenId)}`
  }, unmarshalToken);

  /**
   * Delete the token associated with the specified token ID.
   *
   * @param request - The request {@link DeleteTokenRequest}
   */
  deleteToken = request => this.client.fetch({
    method: 'DELETE',
    path: `/cockpit/v1beta1/tokens/${validatePathParam('tokenId', request.tokenId)}`
  });

  /**
   * Create a contact point to receive alerts for the default receiver.
   *
   * @param request - The request {@link CreateContactPointRequest}
   * @returns A Promise of ContactPoint
   */
  createContactPoint = request => this.client.fetch({
    body: JSON.stringify(marshalCreateContactPointRequest(request, this.client.settings)),
    headers: jsonContentHeaders,
    method: 'POST',
    path: `/cockpit/v1beta1/contact-points`
  }, unmarshalContactPoint);
  pageOfListContactPoints = (() => {
    var _this9 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return _this9.client.fetch({
        method: 'GET',
        path: `/cockpit/v1beta1/contact-points`,
        urlParams: urlParams(['page', request.page], ['page_size', request.pageSize ?? _this9.client.settings.defaultPageSize], ['project_id', request.projectId ?? _this9.client.settings.defaultProjectId])
      }, unmarshalListContactPointsResponse);
    };
  })();

  /**
   * Get a list of contact points for the Cockpit associated with the specified
   * Project ID.
   *
   * @param request - The request {@link ListContactPointsRequest}
   * @returns A Promise of ListContactPointsResponse
   */
  listContactPoints = (() => {
    var _this10 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return enrichForPagination('contactPoints', _this10.pageOfListContactPoints, request);
    };
  })();

  /**
   * Delete a contact point for the default receiver.
   *
   * @param request - The request {@link DeleteContactPointRequest}
   */
  deleteContactPoint = request => this.client.fetch({
    body: JSON.stringify(marshalDeleteContactPointRequest(request, this.client.settings)),
    headers: jsonContentHeaders,
    method: 'POST',
    path: `/cockpit/v1beta1/delete-contact-point`
  });

  /**
   * Enable the sending of managed alerts for the specified Project's Cockpit.
   *
   * @param request - The request {@link EnableManagedAlertsRequest}
   */
  enableManagedAlerts = (() => {
    var _this11 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return _this11.client.fetch({
        body: JSON.stringify(marshalEnableManagedAlertsRequest(request, _this11.client.settings)),
        headers: jsonContentHeaders,
        method: 'POST',
        path: `/cockpit/v1beta1/enable-managed-alerts`
      });
    };
  })();

  /**
   * Disable the sending of managed alerts for the specified Project's Cockpit.
   *
   * @param request - The request {@link DisableManagedAlertsRequest}
   */
  disableManagedAlerts = (() => {
    var _this12 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return _this12.client.fetch({
        body: JSON.stringify(marshalDisableManagedAlertsRequest(request, _this12.client.settings)),
        headers: jsonContentHeaders,
        method: 'POST',
        path: `/cockpit/v1beta1/disable-managed-alerts`
      });
    };
  })();

  /**
   * Trigger a test alert to all of the Cockpit's receivers.
   *
   * @param request - The request {@link TriggerTestAlertRequest}
   */
  triggerTestAlert = (() => {
    var _this13 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return _this13.client.fetch({
        body: JSON.stringify(marshalTriggerTestAlertRequest(request, _this13.client.settings)),
        headers: jsonContentHeaders,
        method: 'POST',
        path: `/cockpit/v1beta1/trigger-test-alert`
      });
    };
  })();

  /**
   * Create a Grafana user for your Cockpit's Grafana instance. Make sure you
   * save the automatically-generated password and the Grafana user ID.
   *
   * @param request - The request {@link CreateGrafanaUserRequest}
   * @returns A Promise of GrafanaUser
   */
  createGrafanaUser = request => this.client.fetch({
    body: JSON.stringify(marshalCreateGrafanaUserRequest(request, this.client.settings)),
    headers: jsonContentHeaders,
    method: 'POST',
    path: `/cockpit/v1beta1/grafana-users`
  }, unmarshalGrafanaUser);
  pageOfListGrafanaUsers = (() => {
    var _this14 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return _this14.client.fetch({
        method: 'GET',
        path: `/cockpit/v1beta1/grafana-users`,
        urlParams: urlParams(['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? _this14.client.settings.defaultPageSize], ['project_id', request.projectId ?? _this14.client.settings.defaultProjectId])
      }, unmarshalListGrafanaUsersResponse);
    };
  })();

  /**
   * Get a list of Grafana users who are able to connect to the Cockpit's
   * Grafana instance.
   *
   * @param request - The request {@link ListGrafanaUsersRequest}
   * @returns A Promise of ListGrafanaUsersResponse
   */
  listGrafanaUsers = (() => {
    var _this15 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return enrichForPagination('grafanaUsers', _this15.pageOfListGrafanaUsers, request);
    };
  })();

  /**
   * Delete a Grafana user from a Grafana instance, specified by the Cockpit's
   * Project ID and the Grafana user ID.
   *
   * @param request - The request {@link DeleteGrafanaUserRequest}
   */
  deleteGrafanaUser = request => this.client.fetch({
    body: JSON.stringify(marshalDeleteGrafanaUserRequest(request, this.client.settings)),
    headers: jsonContentHeaders,
    method: 'POST',
    path: `/cockpit/v1beta1/grafana-users/${validatePathParam('grafanaUserId', request.grafanaUserId)}/delete`
  });

  /**
   * Reset a Grafana user's password specified by the Cockpit's Project ID and
   * the Grafana user ID.
   *
   * @param request - The request {@link ResetGrafanaUserPasswordRequest}
   * @returns A Promise of GrafanaUser
   */
  resetGrafanaUserPassword = request => this.client.fetch({
    body: JSON.stringify(marshalResetGrafanaUserPasswordRequest(request, this.client.settings)),
    headers: jsonContentHeaders,
    method: 'POST',
    path: `/cockpit/v1beta1/grafana-users/${validatePathParam('grafanaUserId', request.grafanaUserId)}/reset-password`
  }, unmarshalGrafanaUser);
  pageOfListPlans = (() => {
    var _this16 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return _this16.client.fetch({
        method: 'GET',
        path: `/cockpit/v1beta1/plans`,
        urlParams: urlParams(['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? _this16.client.settings.defaultPageSize])
      }, unmarshalListPlansResponse);
    };
  })();

  /**
   * Get a list of all pricing plans available.
   *
   * @param request - The request {@link ListPlansRequest}
   * @returns A Promise of ListPlansResponse
   */
  listPlans = (() => {
    var _this17 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return enrichForPagination('plans', _this17.pageOfListPlans, request);
    };
  })();

  /**
   * Select your chosen pricing plan for your Cockpit, specifying the Cockpit's
   * Project ID and the pricing plan's ID in the request.
   *
   * @param request - The request {@link SelectPlanRequest}
   * @returns A Promise of SelectPlanResponse
   */
  selectPlan = request => this.client.fetch({
    body: JSON.stringify(marshalSelectPlanRequest(request, this.client.settings)),
    headers: jsonContentHeaders,
    method: 'POST',
    path: `/cockpit/v1beta1/select-plan`
  }, unmarshalSelectPlanResponse);
}

export { API };
