import { waitForResource } from '../../../internal/async/interval-retrier.js';
import { API as API$1 } from '../../../scw/api.js';
import { validatePathParam, urlParams } from '../../../helpers/marshalling.js';
import { enrichForPagination } from '../../../scw/fetch/resource-paginator.js';
import { PIN_TRANSIENT_STATUSES } from './content.gen.js';
import { marshalCreateVolumeRequest, unmarshalVolume, unmarshalListVolumesResponse, marshalUpdateVolumeRequest, marshalCreatePinByURLRequest, unmarshalPin, marshalCreatePinByCIDRequest, marshalReplacePinRequest, unmarshalReplacePinResponse, unmarshalListPinsResponse } from './marshalling.gen.js';

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const jsonContentHeaders = {
  'Content-Type': 'application/json; charset=utf-8'
};

/** IPFS Pinning service API. */
class API extends API$1 {
  /** Lists the available regions of the API. */
  static LOCALITIES = ['fr-par', 'nl-ams', 'pl-waw'];

  /**
   * Create a new volume. Create a new volume from a Project ID. Volume is
   * identified by an ID and used to host pin references. Volume is personal (at
   * least to your organization) even if IPFS blocks and CID are available to
   * anyone. Should be the first command you made because every pin must be
   * attached to a volume.
   *
   * @param request - The request {@link CreateVolumeRequest}
   * @returns A Promise of Volume
   */
  createVolume = request => this.client.fetch({
    body: JSON.stringify(marshalCreateVolumeRequest(request, this.client.settings)),
    headers: jsonContentHeaders,
    method: 'POST',
    path: `/ipfs/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/volumes`
  }, unmarshalVolume);

  /**
   * Get information about a volume. Retrieve information about a specific
   * volume.
   *
   * @param request - The request {@link GetVolumeRequest}
   * @returns A Promise of Volume
   */
  getVolume = request => this.client.fetch({
    method: 'GET',
    path: `/ipfs/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/volumes/${validatePathParam('volumeId', request.volumeId)}`
  }, unmarshalVolume);
  pageOfListVolumes = (() => {
    var _this = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return _this.client.fetch({
        method: 'GET',
        path: `/ipfs/v1alpha1/regions/${validatePathParam('region', request.region ?? _this.client.settings.defaultRegion)}/volumes`,
        urlParams: urlParams(['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? _this.client.settings.defaultPageSize], ['project_id', request.projectId ?? _this.client.settings.defaultProjectId])
      }, unmarshalListVolumesResponse);
    };
  })();

  /**
   * List all volumes by a Project ID. Retrieve information about all volumes
   * from a Project ID.
   *
   * @param request - The request {@link ListVolumesRequest}
   * @returns A Promise of ListVolumesResponse
   */
  listVolumes = (() => {
    var _this2 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return enrichForPagination('volumes', _this2.pageOfListVolumes, request);
    };
  })();

  /**
   * Update volume information. Update volume information (tag, name...).
   *
   * @param request - The request {@link UpdateVolumeRequest}
   * @returns A Promise of Volume
   */
  updateVolume = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateVolumeRequest(request, this.client.settings)),
    headers: jsonContentHeaders,
    method: 'PATCH',
    path: `/ipfs/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/volumes/${validatePathParam('volumeId', request.volumeId)}`
  }, unmarshalVolume);

  /**
   * Delete an existing volume. Delete a volume by its ID and every pin attached
   * to this volume. This process can take a while to conclude, depending on the
   * size of your pinned content.
   *
   * @param request - The request {@link DeleteVolumeRequest}
   */
  deleteVolume = request => this.client.fetch({
    method: 'DELETE',
    path: `/ipfs/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/volumes/${validatePathParam('volumeId', request.volumeId)}`
  });

  /**
   * Create a pin by URL. Will fetch and store the content pointed by the
   * provided URL. The content must be available on the public IPFS network. The
   * content (IPFS blocks) will be host by the pinning service until pin
   * deletion. From that point, any other IPFS peer can fetch and host your
   * content: Make sure to pin public or encrypted content. Many pin requests
   * (from different users) can target the same CID. A pin is defined by its ID
   * (UUID), its status (queued, pinning, pinned or failed) and target CID.
   *
   * @param request - The request {@link CreatePinByURLRequest}
   * @returns A Promise of Pin
   */
  createPinByURL = request => this.client.fetch({
    body: JSON.stringify(marshalCreatePinByURLRequest(request, this.client.settings)),
    headers: jsonContentHeaders,
    method: 'POST',
    path: `/ipfs/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/pins/create-by-url`
  }, unmarshalPin);

  /**
   * Create a pin by CID. Will fetch and store the content pointed by the
   * provided CID. The content must be available on the public IPFS network. The
   * content (IPFS blocks) will be host by the pinning service until pin
   * deletion. From that point, any other IPFS peer can fetch and host your
   * content: Make sure to pin public or encrypted content. Many pin requests
   * (from different users) can target the same CID. A pin is defined by its ID
   * (UUID), its status (queued, pinning, pinned or failed) and target CID.
   *
   * @param request - The request {@link CreatePinByCIDRequest}
   * @returns A Promise of Pin
   */
  createPinByCID = request => this.client.fetch({
    body: JSON.stringify(marshalCreatePinByCIDRequest(request, this.client.settings)),
    headers: jsonContentHeaders,
    method: 'POST',
    path: `/ipfs/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/pins/create-by-cid`
  }, unmarshalPin);
  replacePin = request => this.client.fetch({
    body: JSON.stringify(marshalReplacePinRequest(request, this.client.settings)),
    headers: jsonContentHeaders,
    method: 'POST',
    path: `/ipfs/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/pins/${validatePathParam('pinId', request.pinId)}/replace`
  }, unmarshalReplacePinResponse);

  /**
   * Get pin information. Retrieve information about the provided **pin ID**,
   * such as status, last modification, and CID.
   *
   * @param request - The request {@link GetPinRequest}
   * @returns A Promise of Pin
   */
  getPin = request => this.client.fetch({
    method: 'GET',
    path: `/ipfs/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/pins/${validatePathParam('pinId', request.pinId)}`,
    urlParams: urlParams(['volume_id', request.volumeId])
  }, unmarshalPin);

  /**
   * Waits for {@link Pin} to be in a final state.
   *
   * @param request - The request {@link GetPinRequest}
   * @param options - The waiting options
   * @returns A Promise of Pin
   */
  waitForPin = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!PIN_TRANSIENT_STATUSES.includes(res.status))), this.getPin, request, options);
  pageOfListPins = request => this.client.fetch({
    method: 'GET',
    path: `/ipfs/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/pins`,
    urlParams: urlParams(['order_by', request.orderBy], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId], ['status', request.status], ['volume_id', request.volumeId])
  }, unmarshalListPinsResponse);

  /**
   * List all pins within a volume. Retrieve information about all pins within a
   * volume.
   *
   * @param request - The request {@link ListPinsRequest}
   * @returns A Promise of ListPinsResponse
   */
  listPins = request => enrichForPagination('pins', this.pageOfListPins, request);

  /**
   * Create an unpin request. An unpin request means that you no longer own the
   * content. This content can therefore be removed and no longer provided on
   * the IPFS network.
   *
   * @param request - The request {@link DeletePinRequest}
   */
  deletePin = request => this.client.fetch({
    method: 'DELETE',
    path: `/ipfs/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/pins/${validatePathParam('pinId', request.pinId)}`,
    urlParams: urlParams(['volume_id', request.volumeId])
  });
}

export { API };
