/**
 * Adds an header to a request through an interceptor.
 *
 * @param key - The header key
 * @param value - The header value
 * @returns The Request interceptor
 *
 * @internal
 */
const addHeaderInterceptor = (key, value) => _ref => {
  let {
    request
  } = _ref;
  const clone = request.clone();
  if (value !== undefined) {
    clone.headers.append(key, value);
  }
  return clone;
};

/**
 * Adds asynchronously an header to a request through an interceptor.
 *
 * @param key - The header key
 * @param value - The header value as a Promise
 * @returns The Request interceptor
 *
 * @internal
 */
const addAsyncHeaderInterceptor = (key, getter) => async request => addHeaderInterceptor(key, await getter())(request);

export { addAsyncHeaderInterceptor, addHeaderInterceptor };
