const isAccessKeyRegex = /^SCW[A-Z0-9]{17}$/i;
const isRegionRegex = /^[a-z]{2}-[a-z]{3}$/i;
const isUUIDRegex = /^[0-9a-fA-F]{8}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{12}$/i;
const isZoneRegex = /^[a-z]{2}-[a-z]{3}-[1-9]$/i;

/** Returns true if the given string has a valid UUID format. */
const isUUID = str => isUUIDRegex.test(str);

/** Returns true if the given string has a valid Scaleway access key format. */
const isAccessKey = str => isAccessKeyRegex.test(str);

/** Returns true if the given string has a valid Scaleway secret key format. */
const isSecretKey = str => isUUID(str);

/** Returns true if the given string has a valid Scaleway organization ID format. */
const isOrganizationId = str => isUUID(str);

/** Returns true if the given string has a valid Scaleway project ID format. */
const isProjectId = str => isUUID(str);

/** Returns true if the given string has a valid region format. */
const isRegion = str => isRegionRegex.test(str);

/** Returns true if the given string has a valid zone format. */
const isZone = str => isZoneRegex.test(str);

/** Returns true if the given string has a valid URL format and starts by `http(s):`. */
const isURL = str => {
  let url;
  try {
    url = new URL(str);
  } catch {
    return false;
  }
  return url.protocol === 'http:' || url.protocol === 'https:';
};

export { isAccessKey, isOrganizationId, isProjectId, isRegion, isSecretKey, isURL, isUUID, isZone };
